package ai;

import common.ConfigurationTTool;
import launcher.LauncherException;
import launcher.RshClient;
import launcher.RshClientReader;
import myutil.TraceManager;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Class AIUseCaseDiagram
 * <p>
 * Creation: 18/03/2024
 *
 * @author Alan Birchler De Allende
 */
public class CAPECTracer extends AIInteract {
    public static String QUESTIONTRACECAPECS = "From the provided system specifications, identify all of the possible " +
            "attack patterns that an attacker could use to exploit the system.";

    private String command;
    private Path projectPath;

    public CAPECTracer(AIChatData _chatData) {
        super(_chatData);
        projectPath = Paths.get("../capectracer").normalize().toAbsolutePath();
        command = getCapecCommand(projectPath.toString());
    }

    @Override
    public void internalRequest() {
        chatData.feedback.addToChat(QUESTIONTRACECAPECS, true);
        String systemSpec = chatData.lastQuestion.trim();

        writeToFile(projectPath + "/system_specs.txt", systemSpec);

        String results = runCapecTracer(projectPath.toString());
        chatData.feedback.addToChat(results, false);
    }

    public String getInfo() {
        return "Running command: " + command;
    }

    private void writeToFile(String filePath, String content) {
        try {
            // Create a FileWriter object
            FileWriter fileWriter = new FileWriter(filePath, false);

            // Wrap the FileWriter in a BufferedWriter for efficient writing
            BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

            // Write the content to the file
            bufferedWriter.write(content);

            // Close the BufferedWriter to flush the buffer and close the file
            bufferedWriter.close();

        } catch (IOException e) {
            TraceManager.addDev(e.getMessage());
        }
    }

    private String getCapecCommand(String capecTracerFolder) {
        String python = "python3";
        if (ConfigurationTTool.PythonPathForCapec.length() > 0) {
            python = ConfigurationTTool.PythonPathForCapec;
        }
        return python + " " + capecTracerFolder + "/capec_tracer.py";
    }

    private String runCapecTracer(String capecTracerFolder) {
        String traces = "";

        try {
            RshClient rshc = new RshClient("localhost");
            rshc.setCmd(command);
            rshc.sendExecuteCommandRequest();
            RshClientReader data = rshc.getDataReaderFromProcess();
            int characterInt = data.read();
            StringBuilder output = new StringBuilder();

            while (characterInt != -1) {
                output.append((char) characterInt);
                characterInt = data.read();
            }

            if (!output.toString().contains("Error")) {
                byte[] bytes = Files.readAllBytes(Path.of(capecTracerFolder + "/traced_capecs.txt"));
                traces = new String(bytes);
            }
        } catch (IOException | LauncherException e) {
            TraceManager.addDev(e.getMessage());
        }

        if (traces.isEmpty()) {
            return "The tracer failed to run successfully.";
        }
        else {
            return traces;
        }
    }

    @Override
    public Object applyAnswer(Object input) {
        return null;
    }
}
